      SUBROUTINE SECTIO(AX,AY,BX,BY,IM,JM,KB,H,U,V,WR,S,T, &
     &           DX,DY,DZ,USEC,UTSEC,WSEC,SSEC,RHOSEC,SECDEP,SECLEN,MAXSEC, &
     &           NSEC,IFAIL)
!***BEGIN PROLOGUE SECTIO
!***DATE WRITTEN   910426 (YYMMDD)
!***REVISION DATE  920831 (YYMMDD)
!***AUTHOR
!            Jarle Berntsen, Institute of Marine Research,
!            Postboks 1870,
!            N-5024 Bergen-Nordnes, Norway
!            Email..  jarleb@imr.no
!
!            A great portion of the program is based
!            on an earlier flux routine by B.Aadlandsvik
!
!***PURPOSE SECTIO computes the velocities 
!           through a given section.
!
!           This version of SECTIO is based on the
!           datastructures used in the ECOM3D model.
!           SECTIO assumes constant grid spacing.
!           That is DX and DY must both be constant.
!
!***DESCRIPTION
!
!           A section is drawn between the point A with
!           coordinates (AX,AY) and the point B with
!           coordinates (BX,BY).
!           0 <= AX,BX <= IM and 0 <= AY,BY <= JM.
!
!           SECTIO computes the velocities normal to the section
!           from A to B. 
!
!           The section is parameterized as follows 
!
!           (1 - ST)*(AX,AY) + ST*(BX,BY) , 0 <= ST <= 1.
!
!           The routine computes all values of the parameter
!           ST for which the section intersects the
!           horizontal gridnet.
!           We then loop over all horizontal subsections
!           and all vertical layers and compute the velocities
!           normal to the section, the vertical velocity,
!           the salinity, the temperature,
!           the length of the section and the depth of the
!           section.
!
!   ON ENTRY
!
!     AX     Real.
!            X-Coordinate of point A.
!     AY     Real.
!            Y-Coordinate of point A.
!     BX     Real.
!            X-Coordinate of point B.
!     BY     Real.
!            Y-Coordinate of point B.
!     IM   Integer.
!            Number of cells in X-direction.
!     JM   Integer.
!            Number of cells in Y-direction.
!     KB   Integer.
!            Number of cells in Sigma-direction.
!     H      Real array of dimension (IM,JM).
!            The bottom depth (m).
!     U      Real array of dimension (IM,JM,KB).
!            Horizontal velocity in X-direction (m/s).
!     V      Real array of dimension (IM,JM,KB). 
!            Horizontal velocity in Y-direction (m/s).
!     WR     Real array of dimension (IM,JM,KB).
!            Vertical velocity (m/s).
!     S      Real array of dimension (IM,JM,KB).
!            Salinity.
!     T      Real array of dimension (IM,JM,KB).
!            Temperature.
!     DX     Real.
!            The width of a cell in X-direction (m).
!     DY     Real.
!            The width of a cell in Y-direction (m).
!     DZ     Real array of dimension KB.
!            The sigma coordinate thickness of the layers.
!     MAXSEC Integer.
!            The maximum allowed number of subsections.
!            MAXSEC must be greater or equal to NSEC.
!            If MAXSEC is selected to be equal to IM+JM,
!            MAXSEC will be great enough for all sections.
!     
!   ON RETURN
!
!     USEC   Real array of dimension (MAXSEC,KB).
!            The horisontal velocities normal to the section.
!     UTSEC  Real array of dimension (MAXSEC,KB).
!            The horisontal velocities tangential to the section.
!     WSEC   Real array of dimension (MAXSEC,KB).
!            The vertical velocities of the section.
!     SSEC   Real array of dimension (MAXSEC,KB).
!            The salinities of the section.
!     SECDEP Real array of dimension MAXSEC.
!            The depths of each subsection.
!     SECLEN Real array of dimension MAXSEC.
!            The lengths of each subsection.
!     NSEC   Integer.
!            The number of sea subsections.
!     IFAIL  Integer.
!            IFAIL is 0 on exit if FLUXR and FLUXL have been
!            successfully computed.
!            IFAIL is 1 if IM, JM or KB <= 0 on entry.
!            IFAIL is 2 if (AX,AY) = (BX,BY) on entry.
!            IFAIL is 3 if A or B is outside the region of integration.
!            IFAIL is 4 if we try to compute the velocities for
!                  NSEC > MAXSEC.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE SECTIO
!
!   Global variables.
!
IMPLICIT NONE
      INTEGER IM,JM,KB,MAXSEC,NSEC,IFAIL
      REAL AX,AY,BX,BY,H(IM,JM),U(IM,JM,KB), &
     &     V(IM,JM,KB),DZ(KB),USEC(MAXSEC,KB),UTSEC(MAXSEC,KB), &
     &     WSEC(MAXSEC,KB),SSEC(MAXSEC,KB),RHOSEC(MAXSEC,KB), &
     &     SECLEN(MAXSEC),SECDEP(MAXSEC), &
     &     DX,DY,WR(IM,JM,KB),S(IM,JM,KB),T(IM,JM,KB)
!
!   Local variables.
!
!   MAXCO  Maximum number of gridcell intersections that may be
!          crossed by our section.
!   IXCO   Number of gridcell intersections that is crossed
!          by our section in X-direction.
!   IYCO   Number of gridcell intersections that is crossed
!          by our section in Y-direction.
!   SX     Values of the parameter ST for which our section
!          crosses the gridcell intersections in X-direction.
!   SY     Values of the parameter ST for which our section 
!          crosses the gridcell intersections in Y-direction.
!   ST     Sorted values of the parameter ST for which our section
!          crosses the horisontal grid.
!   RGREAT A real parameter that must be greater than MAX(IM,JM).
!   ICOUNT The number of intercell subsections.
!   ALPHA  The direction of the section.
!   LENGTH Horizontal length of a subsection.
!   SMID   ST value at the midpoint of a subsection.
!   XMID   X-Coordinate at the midpoint of a subsection.
!   YMID   Y-Coordinate at the midpoint of a subsection.
!   ICELL  Index in X-direction of the cell containing the midpoint.
!   JCELL  Index in Y-direction of the cell containing the midpoint.
!   UMID   U value at the midpoint of a subsection.
!   VMID   V value at the midpoint of a subsection.
!   VELOCI Total horizontal velocity.
!   BETA   Direction of the flow.
!
      INTEGER MAXCO,IXCO,I,IYCO,ICOUNT,IXPEK,IYPEK,ICELL, &
     &        JCELL,L,K
      PARAMETER (MAXCO = 1000)
      REAL SX(MAXCO),SY(MAXCO),ST(0:MAXCO),RGREAT
      PARAMETER (RGREAT = 1.E12)
      REAL SLEN,ALPHA,SMID,XMID,YMID,PARTX,PARTY,SDEP, &
     &     LENGTH,UMID,VMID,VELOCI,BETA
!
!***FIRST EXECUTABLE STATEMENT SECTIO
!
!
!   Test on legal input.
!
      IF (IM.LE.0) THEN
        IFAIL = 1
        RETURN
      ENDIF
      IF (JM.LE.0) THEN
        IFAIL = 1
        RETURN
      ENDIF
      IF (KB.LE.0) THEN
        IFAIL = 1
        RETURN
      ENDIF
!
!   Test on A = B.
!
      IF ((AX.EQ.BX).AND.(AY.EQ.BY)) THEN
        IFAIL = 2
        RETURN
      ENDIF
!
!   Test on A or B outside the region.
!
      IF ((AX.LT.0.).OR.(AX.GT.FLOAT(IM)).OR. &
     &    (BX.LT.0.).OR.(BX.GT.FLOAT(IM)).OR. &
     &    (AY.LT.0.).OR.(AY.GT.FLOAT(JM)).OR. &
     &    (BY.LT.0.).OR.(BY.GT.FLOAT(JM))) THEN
        IFAIL = 3
        RETURN
      ENDIF
!
!   Locate the values of the parameter ST for which our section
!   crosses the gridcell intersections in X-direction.
!   Place these parameters sorted in SX.
!
      IXCO = 0
!
!   If AX < BX, then search to the right.
!
      IF (AX.LT.BX) THEN
        I = INT(AX) + 1
10      IF (FLOAT(I).LT.BX) THEN
          IXCO = IXCO + 1
          SX(IXCO) = (I-AX)/(BX-AX)
          I = I + 1
          GO TO 10
        ENDIF
!
!   If AX > BX, then search to the left.
!
      ELSEIF (AX.GT.BX) THEN
        I = INT(AX)
        IF (AX.EQ.FLOAT(I)) THEN
          I = I - 1
        ENDIF
20      IF (FLOAT(I).GT.BX) THEN
          IXCO = IXCO + 1
          SX(IXCO) = (I-AX)/(BX-AX)
          I = I - 1
          GO TO 20
        ENDIF
!
!   If AX = BX, then IXCO = 0.
!
      ELSEIF (AX.EQ.BX) THEN
        IXCO = 0
      ENDIF
!
!   Locate the values of the parameter ST for which our section
!   crosses the gridcell intersections in Y-direction.
!   Place these parameters sorted in SY.
!
      IYCO = 0
!
!   If AY < BY, then search to the north.
!
      IF (AY.LT.BY) THEN
        I = INT(AY) + 1
30      IF (FLOAT(I).LT.BY) THEN
          IYCO = IYCO + 1
          SY(IYCO) = (I-AY)/(BY-AY)
          I = I + 1
          GO TO 30
        ENDIF
!
!   If AY > BY, then search to the south.
!
      ELSEIF (AY.GT.BY) THEN
        I = INT(AY)
        IF (AY.EQ.FLOAT(I)) THEN
          I = I - 1
        ENDIF
40      IF (FLOAT(I).GT.BY) THEN
          IYCO = IYCO + 1
          SY(IYCO) = (I-AY)/(BY-AY)
          I = I - 1
          GO TO 40
        ENDIF
!
!   If AY = BY, then IYCO = 0.
!
      ELSEIF (AY.EQ.BY) THEN
        IYCO = 0
      ENDIF
!
!   Merge SX and SY into the array ST and include the endpoints.
!   ICOUNT is the number of intercell subsections.
!
      SX(IXCO+1) = RGREAT
      SY(IYCO+1) = RGREAT
      IXPEK = 1
      IYPEK = 1
      ICOUNT = 0
      ST(ICOUNT) = 0.0
!
!   If ST(ICOUNT) < RGREAT, we have still not reached the end
!   of the section.
!
100   IF (ST(ICOUNT).LT.RGREAT) THEN
        ICOUNT = ICOUNT + 1
!
!   If SX(IXPEK) < SY(IYPEK), we cross the Y-axis before
!   the X-axis.
!
        IF (SX(IXPEK).LT.SY(IYPEK)) THEN
          ST(ICOUNT) = SX(IXPEK)
          IXPEK = IXPEK + 1
! 
!   If SX(IXPEK) > SY(IYPEK), we cross the X-axis before 
!   the Y-axis. 
!
        ELSEIF(SX(IXPEK).GT.SY(IYPEK)) THEN
          ST(ICOUNT) = SY(IYPEK)
          IYPEK = IYPEK + 1
!  
!   If SX(IXPEK) = SY(IYPEK), we cross the X-axis and the
!   Y-axis at the same point.  
!   This is a corner of a gridcell.
!
        ELSEIF(SX(IXPEK).EQ.SY(IYPEK)) THEN
          ST(ICOUNT) = SY(IYPEK)
          IYPEK = IYPEK + 1
          IXPEK = IXPEK + 1
        ENDIF
        GO TO 100
      ENDIF
!
!   Replace the last value in ST (RGREAT), with the value
!   that makes the section end at B.
!
      ST(ICOUNT) = 1.0
!
!   Initate NSEC.
!
      NSEC = 0
      SLEN = SQRT((DX*(BX-AX))**2 + (DY*(BY-AY))**2)
      IF ((BY-AY).NE.0. .OR. (BX-AX).NE.0. ) THEN
        ALPHA = ATAN2(DY*(BY-AY),DX*(BX-AX))
      ELSE
        ALPHA = 0.
      ENDIF
!
!   Loop over all subsections from A to B.
!
      DO 500 L = 1,ICOUNT
        LENGTH = SLEN*(ST(L) - ST(L-1))
        SMID = 0.5*(ST(L-1) + ST(L))
        XMID = (1.0 - SMID)*AX + SMID*BX
        YMID = (1.0 - SMID)*AY + SMID*BY
        ICELL = INT(XMID) + 1
        JCELL = INT(YMID) + 1
        IF (XMID.EQ.FLOAT(IM)) THEN
          ICELL = IM
        ENDIF
        IF (YMID.EQ.FLOAT(JM)) THEN
          JCELL = JM
        ENDIF
        PARTX = XMID - (ICELL - 1)
        PARTY = YMID - (JCELL - 1)
        SDEP = H(ICELL,JCELL)
        IF (PARTX.EQ.1. .AND. ICELL.LT.IM) THEN
          SDEP = 0.5*(H(ICELL,JCELL)+H(ICELL+1,JCELL))
        ENDIF
        IF (PARTX.EQ.0. .AND. ICELL.GT.1) THEN 
          SDEP = 0.5*(H(ICELL,JCELL)+H(ICELL-1,JCELL)) 
        ENDIF
        IF (PARTY.EQ.1. .AND. JCELL.LT.JM) THEN 
          SDEP = 0.5*(H(ICELL,JCELL)+H(ICELL,JCELL+1)) 
        ENDIF 
        IF (PARTY.EQ.0. .AND. JCELL.GT.1) THEN  
          SDEP = 0.5*(H(ICELL,JCELL)+H(ICELL,JCELL-1))  
        ENDIF
!
!   If (ICELL,JCELL) is a sea cell, we loop 
!   over all layers.
!
        IF (SDEP.GE.10. .AND. LENGTH.GT.1E-6) THEN
          NSEC = NSEC + 1
          IF (NSEC.GT.MAXSEC) THEN
            IFAIL = 4
            RETURN
          ENDIF
          SECDEP(NSEC) = SDEP
          SECLEN(NSEC) = LENGTH
          DO 250 K = 1,KB
            IF (ICELL.LT.IM) THEN
              UMID = (1.0 - PARTX)*U(ICELL,JCELL,K) + &
     &             PARTX*U(ICELL+1,JCELL,K)
            ELSE
              UMID = U(ICELL,JCELL,K)
            ENDIF
            IF (JCELL.LT.JM) THEN
              VMID = (1.0 - PARTY)*V(ICELL,JCELL,K) + &
     &             PARTY*V(ICELL,JCELL+1,K)
            ELSE
              VMID = V(ICELL,JCELL,K)
            ENDIF
            VELOCI = SQRT(UMID**2 + VMID**2)
!
!   Compute the direction of the flow.
!
            IF ((UMID.EQ.0.).AND.(VMID.EQ.0.)) THEN
              BETA = 0.
            ELSE
              BETA = ATAN2(VMID,UMID)
            ENDIF
!
!   Compute the velocities, salinities and
!   temperatures of this section.
!
            USEC(NSEC,K) = VELOCI*SIN(ALPHA-BETA)
            UTSEC(NSEC,K) = VELOCI*COS(ALPHA-BETA)
            WSEC(NSEC,K) = WR(ICELL,JCELL,K)
            SSEC(NSEC,K) = SQRT(USEC(NSEC,K)**2 + UTSEC(NSEC,K)**2 )
            RHOSEC(NSEC,K) = T(ICELL,JCELL,K) - 1000.
250       CONTINUE
        ENDIF
!
!   Continue to next horizontal cell.
!
500   CONTINUE
!
!***END SECTIO
!
      RETURN
      END
