SUBROUTINE SUPERBEEF(FIELD,FAVER,ICODE)
!
!           SUPERBEEF integrates the equation
!
!           d(FIELD*D)/d(T) + d(FIELD*U*D)/d(X) + d(FIELD*V*D)/d(Y) +
!       
!           d(FIELD*W)/d(SIGMA) = d(AH*d(FIELD*D)/d(X))/d(X) +
!                                 d(AH*d(FIELD*D)/d(Y))/d(Y)
!
!           from timestep n to timestep n+1
!           with a superbee limiter TVD scheme.
!
!           d(*) is the differential operator,
!           FIELD is the scalar field to be integrated,
!           D is the dynamic depth,
!           T is time,
!           X and Y are the horisontal coordinates,
!           U is the horisontal velocity in X-direction,
!           V is the horisontal velocity in Y-direction,
!           SIGMA is the vertical coordinate,
!           W is the vertical velocity. 
!           
!           FAVER is the average value of the scalar field which is 
!                 subtracted from FIELD before differences are 
!                 computed to reduce the cancellation errors.
!           ICODE is an integer flag to signal which scalar field
!                 that is advected. ICODE is needed when river-runoff
!                 is included.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
  USE MOD_BOUND
  IMPLICIT NONE
  REAL FIELD(IM,JM,KB),FAVER
  INTEGER ICODE
!
!   Local variables.
!
  REAL, DIMENSION(IM,JM,KB) :: XFLUX,YFLUX,ZFLUX
  REAL :: xfluxup,YFLUXUP,ZFLUXUP
  REAL :: xfluxlw,YFLUXLW,ZFLUXLW
  REAL :: XFLUXD,YFLUXD
  REAL AREA,NETFLUX,VOLUME,VOLUMEP,LAMBDAX,LAMBDAY
  REAL R,PHIR,NUMER,DENUM,EPSILON,AREAT,AREATU,AREATV,AREATW
  INTEGER I,J,K,NSIG,pt
  REAL FRAC,FLUX,FIELDRIVER
!
  LAMBDAX = DT/DX
  LAMBDAY = DT/DY
  EPSILON = 1.E-10
!
!   Define FIELD(KB) = FIELD(KB-1)
!
  FIELD(1:IM,1:JM,KB) = FIELD(1:IM,1:JM,KB-1)
  FIELD = FIELD - FAVER
!
  XFLUX = 0.
  XFLUXUP = 0.
  XFLUXLW = 0.
  XFLUXD = 0.
  YFLUX = 0.
  YFLUXUP = 0.
  YFLUXLW = 0.
  YFLUXD = 0.
  ZFLUX = 0.
  ZFLUXUP = 0.
  ZFLUXLW = 0.
!
!   Compute F*U*D in U-points of the
!   Arakawa C-grid.
!
!   Compute the sum of the advective and diffusive
!   fluxes  in X-direction.
!
  IF (RIVLOG) THEN
     DO I = 1,NUMC
        IF (ICODE.EQ.1) THEN
!
!  Salinity of river water
!
           FIELDRIVER = SRIVER(I)
        ELSEIF (ICODE.EQ.2) THEN
!
! Temperature of the river water
!
           FIELDRIVER = TRIVER(I)
        ELSE
           WRITE(*,*)'ERROR IN SUPERBEEF'
           STOP
        ENDIF
        FIELDRIVER = FIELDRIVER - FAVER
        DO K = KRIVER1(I),KRIVER2(I)
           FRAC = DZ(K)/DZRIVER(I)
           FLUX = FRAC*FRIVER(I)*FIELDRIVER 
           IF (CODERIVER(I).EQ.1) THEN
              XFLUX(IRIVER(I),JRIVER(I),K) = FLUX
           ELSEIF (CODERIVER(I).EQ.2) THEN
              YFLUX(IRIVER(I),JRIVER(I),K) = FLUX
           ELSEIF (CODERIVER(I).EQ.3) THEN
              XFLUX(IRIVER(I)+1,JRIVER(I),K) = -FLUX
           ELSEIF (CODERIVER(I).EQ.4) THEN
              YFLUX(IRIVER(I),JRIVER(I)+1,K) = -FLUX
           ELSE
              WRITE(*,*)'ERROR IN SUPERBEEF-2'
              STOP
           ENDIF
        END DO
     END DO
  ENDIF

!$OMP PARALLEL DO &
!$OMP PRIVATE(I,J,K,AREAT,AREATU,NSIG,NUMER,DENUM &
!$OMP  ,R,PHIR,XFLUXUP,XFLUXLW,XFLUXD) &
!$OMP firstprivate(dx,dy,lambdax)
  DO pt=1,numup
     i=indu(1,pt)
     j=indu(2,pt)
!DO J = 2,JM-1
!  DO I = 2,IM
!    IF (DUM(I,J).NE.0.) THEN
     DO K = 1,KB-1
        AREAT = DY*DUDZ(I,J,K)
        AREATU = AREAT*UADV(I,J,K)
        XFLUXD = AH(I,J,K)*AREAT*(FIELD(I,J,K)-FIELD(I-1,J,K))/DX
        IF (UADV(I,J,K).GE.0.) THEN
           XFLUXUP = AREATU*FIELD(I-1,J,K)         
           NSIG = 1
        ELSE
           XFLUXUP = AREATU*FIELD(I,J,K)         
           NSIG = -1
        ENDIF
        XFLUXLW = 0.5*AREATU*(FIELD(I,J,K)+FIELD(I-1,J,K)) - &
             & 0.5*LAMBDAX*UADV(I,J,K)**2*(FIELD(I,J,K)-FIELD(I-1,J,K))*AREAT
        IF (I.GT.2 .AND. I.LE.IM-1) THEN
           NUMER = (ABS(UADV(I-NSIG,J,K)) - LAMBDAX*UADV(I-NSIG,J,K)**2)*  &
                &         DY*(DUDZ(I-NSIG,J,K)*(FIELD(I-NSIG,J,K)-FIELD(I-1-NSIG,J,K)))
           DENUM = (ABS(UADV(I,J,K)) - LAMBDAX*UADV(I,J,K)**2)*  &
                &         DY*(DUDZ(I,J,K)*(FIELD(I,J,K)-FIELD(I-1,J,K)))
           IF (ABS(DENUM).GT.EPSILON) THEN
              R = NUMER/DENUM
              PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
           ELSE 
              PHIR = 0.
           ENDIF
        ELSE
           PHIR = 0.
        ENDIF
        XFLUX(I,J,K) = XFLUXUP + PHIR*(XFLUXLW-XFLUXUP)
        XFLUX(I,J,K) = XFLUX(I,J,K) - XFLUXD
     END DO
!    ENDIF
!  END DO
  END DO
!
!   Compute the sum of the advective and diffusive
!   fluxes  in Y-direction.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,AREAT,AREATV,NSIG,NUMER,DENUM,R,PHIR,&
!$OMP yfluxup,yfluxd,yfluxlw) firstprivate(dx,dy,lambday,epsilon)
  DO pt=1,numvp
     i=indv(1,pt)
     j=indv(2,pt)
!DO J = 2,JM
!  DO I = 2,IM-1
!    IF (DVM(I,J).NE.0.) THEN
     DO K = 1,KB-1
        AREAT = DX*DVDZ(I,J,K)
        AREATV = AREAT*VADV(I,J,K)
        YFLUXD = AREAT*AH(I,J,K)*((FIELD(I,J,K)-FIELD(I,J-1,K))/DY)
        IF (VADV(I,J,K).GE.0.) THEN
           YFLUXUP = AREATV*FIELD(I,J-1,K)
           NSIG = 1
        ELSE
           YFLUXUP = AREATV*FIELD(I,J,K)
           NSIG = -1
        ENDIF
        YFLUXLW = 0.5*AREATV*(FIELD(I,J,K)+FIELD(I,J-1,K)) - &
             & 0.5*LAMBDAY*VADV(I,J,K)**2*(FIELD(I,J,K)-FIELD(I,J-1,K))*AREAT
        IF (J.GT.2 .AND. J.LE.JM-1) THEN
           NUMER = (ABS(VADV(I,J-NSIG,K)) - LAMBDAY*VADV(I,J-NSIG,K)**2)*  &
                &         DX*(DVDZ(I,J-NSIG,K)*(FIELD(I,J-NSIG,K)-FIELD(I,J-1-NSIG,K)))
           DENUM = (ABS(VADV(I,J,K)) - LAMBDAY*VADV(I,J,K)**2)*  &
                &         DX*(DVDZ(I,J,K)*(FIELD(I,J,K)-FIELD(I,J-1,K)))
           IF (ABS(DENUM).GT.EPSILON) THEN
              R = NUMER/DENUM
              PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
           ELSE
              PHIR = 0.
           ENDIF
        ELSE
           PHIR = 0.
        ENDIF
        YFLUX(I,J,K) = YFLUXUP + PHIR*(YFLUXLW-YFLUXUP)
        YFLUX(I,J,K) = YFLUX(I,J,K) - YFLUXD
     END DO
!    ENDIF
!  END DO
  END DO
!
!   Compute the advective fluxes in sigma-direction.
!
  AREA = DX*DY

!$OMP PARALLEL DO &
!$OMP PRIVATE(I,J,K,AREATW,NSIG,NUMER,DENUM,R,PHIR,zfluxup,zfluxlw) &
!$OMP FIRSTPRIVATE(AREA,dt,epsilon,dzz)
  DO pt=1,numcp
     i=indc(1,pt)
     j=indc(2,pt)
!DO J = 2,JM-1
!  DO I = 2,IM-1
!    IF(FSM(I,J).NE.0.) THEN
     DO K = 2,KB
        AREATW = AREA*W(I,J,K)
        IF (W(I,J,K).GE.0.) THEN
           ZFLUXUP = AREATW*FIELD(I,J,K)
           NSIG = -1
        ELSE
           ZFLUXUP = AREATW*FIELD(I,J,K-1)
           NSIG = 1
        ENDIF
        ZFLUXLW = 0.5*AREATW*(FIELD(I,J,K-1)+FIELD(I,J,K)) - &
             &    0.5*AREA*DT*W(I,J,K)**2*(FIELD(I,J,K-1)-FIELD(I,J,K))/(DZZ(K-1)*D(I,J))
        IF ( (K.EQ.2 .AND. NSIG.EQ.-1) .OR. &
             &      (K.EQ.KB .AND. NSIG.EQ.1) .OR. &
             &      (K.GE.3 .AND. K.LE.KB-1)) THEN
           NUMER = (ABS(W(I,J,K-NSIG)) - (DT/(DZZ(K-1-NSIG)*D(I,J)))* &
                &         W(I,J,K-NSIG)**2)*  &
                &         (FIELD(I,J,K-1-NSIG)-FIELD(I,J,K-NSIG))
           DENUM = (ABS(W(I,J,K)) - (DT/(DZZ(K-1)*D(I,J)))* &
                &         W(I,J,K)**2)*  &
                &         (FIELD(I,J,K-1)-FIELD(I,J,K))
           IF (ABS(DENUM).GT.EPSILON) THEN
              R = NUMER/DENUM
              PHIR = MAX(0., MIN(2.*R,1.) , MIN(R,2.))
           ELSE
              PHIR = 1.
           ENDIF
        ELSE
           PHIR = 1.
        ENDIF
!
        ZFLUX(I,J,K) = ZFLUXUP + PHIR*(ZFLUXLW-ZFLUXUP)
     END DO
!    ENDIF
!  END DO
  END DO
!
!   Take a step to timestep n+1  by adding up the fluxes in and out of each cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,K,NETFLUX,VOLUME,VOLUMEP) &
!$OMP FIRSTPRIVATE(dx,dy,dz,dt)
  DO pt=1,numcp
     i=indc(1,pt)
     j=indc(2,pt)
!DO J = 2,JM-1
!  DO I =2,IM-1
!    IF (FSM(I,J).GT.0.0) THEN
     DO K = 1,KB-1
        NETFLUX = (ZFLUX(I,J,K)-ZFLUX(I,J,K+1)) + &
             &            XFLUX(I+1,J,K) - XFLUX(I,J,K) + &
             &            YFLUX(I,J+1,K) - YFLUX(I,J,K)
        VOLUME = DX*DY*DZ(K)*(H(I,J)+ETA(I,J))
        VOLUMEP = DX*DY*DZ(K)*(H(I,J)+ETAP(I,J))
        FIELD(I,J,K) = (FIELD(I,J,K)*VOLUMEP - DT*NETFLUX)/VOLUME
!
     END DO
!    ENDIF
!  END DO
  END DO
!
  FIELD = FIELD + FAVER
!
  RETURN
END SUBROUTINE SUPERBEEF
